#ifndef _AUTOCOMPLETE_H_
#define _AUTOCOMPLETE_H_


//// Defines

#define STRID_HOTKEYS                    301
#define STRID_COMPLETEWITHLIST           302
#define STRID_COMPLETENEXT               303
#define STRID_COMPLETEPREV               304
#define STRID_AUTOLIST                   305
#define STRID_AFTERCHAR_PRE              306
#define STRID_AFTERCHAR_POST             307
#define STRID_DOCUMENT                   308
#define STRID_ADDDOCUMENTWORDS           309
#define STRID_COMPLETENONSYNTAXDOCUMENT  310
#define STRID_MAXDOCUMENT                311
#define STRID_CHARS                      312
#define STRID_ADDHIGHLIGHTWORDS          313
#define STRID_RIGHTDELIMITERS            314
#define STRID_SYNTAXDELIMITERS           315

#define DLLA_AUTOCOMPLETE_SET  1

#define SIZE_MINX      30
#define SIZE_MINY      30

//CreateAutoCompleteWindow flags
#define CAW_COMPLETEONE    0x1
#define CAW_COMPLETEEXACT  0x2
#define CAW_COMPLETEWINDOW 0x4
#define CAW_AUTOLIST       0x8

//CreateAutoCompleteWindow return value
#define CAWE_SUCCESS        0
#define CAWE_NOEDITWINDOW   1
#define CAWE_DOCUMENTLIMIT  2
#define CAWE_GETTITLEPART   3
#define CAWE_AUTOLIST       4
#define CAWE_GETBLOCK       5
#define CAWE_OTHER          6

//BLOCKINFO structure types
#define BIT_BLOCK          0
#define BIT_DOCWORD        1
#define BIT_HIGHLIGHT      2

#define AUTOCOMPLETEA   "AutoComplete"
#define AUTOCOMPLETEW  L"AutoComplete"
#define IDC_LIST        1001

#ifndef WM_MOUSEWHEEL
  #define WM_MOUSEWHEEL 0x020A
#endif


//// Structures

typedef struct _BLOCKINFO {
  struct _BLOCKINFO *next;
  struct _BLOCKINFO *prev;
  int nStructType;
  wchar_t wchFirstLowerChar;
  wchar_t *wpTitle;
  int nTitleLen;

  struct _BLOCKINFO *master;
  INT_PTR *firstHandle;
  INT_PTR *lastHandle;
  wchar_t *wpBlock;
  int nBlockLen;
  int nLinesInBlock;
  HSTACK hHotSpotStack;
  INT_PTR nHotSpotBlockBegin;
} BLOCKINFO;

typedef struct _BLOCKINFOHANDLE {
  struct _BLOCKINFOHANDLE *next;
  struct _BLOCKINFOHANDLE *prev;
  BLOCKINFO *lpBlockInfo;
} BLOCKINFOHANDLE;

typedef struct _HOTSPOT {
  struct _HOTSPOT *next;
  struct _HOTSPOT *prev;
  int nHotSpotInitPos;
  int nHotSpotInitLen;
  int nHotSpotPos;
  int nHotSpotLen;
} HOTSPOT;

typedef struct _DOCWORDINFO {
  struct _DOCWORDINFO *next;
  struct _DOCWORDINFO *prev;
  int nStructType;
  wchar_t wchFirstLowerChar;
  wchar_t *wpDocWord;
  int nDocWordLen;
} DOCWORDINFO;

typedef struct {
  DOCWORDINFO *first;
  DOCWORDINFO *last;
  INT_PTR lpSorted[FIRST_NONLATIN + 1];
} HDOCWORDS;


//// Prototypes

BOOL CALLBACK AutoCompleteSetupDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK AutoCompleteEditParentMessages(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK AutoCompleteEditProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK GetMsgProc(int code, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CompleteWithListProc(void *lpParameter);
BOOL CALLBACK CompleteNextProc(void *lpParameter);
BOOL CALLBACK CompletePrevProc(void *lpParameter);
LRESULT CALLBACK AutoCompleteWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK NewListboxProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

//AutoComplete window
DWORD CreateAutoCompleteWindow(HSYNTAXFILE *lpTheme, DWORD dwFlags);
BOOL MoveAutoCompleteWindow();
void CloseAutoCompleteWindow();

//AutoComplete window listbox
void FillListbox(HSYNTAXFILE *lpSyntaxFile, HDOCWORDS *hDocWordsStack, const wchar_t *wpTitlePart);
void SetSelListbox(int nIndex);
BLOCKINFO* GetBlockListbox();

//Scheme
int ParseBlock(HSYNTAXFILE *lpScheme, HSTACK *hHotSpotStack, const wchar_t *wpInput, int nInputLen, wchar_t *wszOutput, int *nOutputLines);

//Title
TITLEINFO* StackInsertTitle(HTITLESTACK *hStack);
void StackFreeTitle(HTITLESTACK *hStack);

//Title part
BOOL GetEditTitlePart(HDELIMSTACK *hDelimiterStack, wchar_t *wszTitle, int nTitleMax, INT_PTR *nMin, INT_PTR *nMax);
void CompleteTitlePart(BLOCKINFO *lpBlockInfo, INT_PTR nMin, INT_PTR nMax);

//Block
BLOCKINFO* StackInsertBlock(HBLOCKSTACK *hBlockStack, wchar_t *wpTitle, int nTitleLen);
BLOCKINFO* StackGetBlock(HSYNTAXFILE *lpSyntaxFile, HDOCWORDS *hDocWordsStack, const wchar_t *wpTitlePart, int nTitlePartLen, BOOL *bOnlyOne);
void StackFreeBlock(HBLOCKSTACK *hBlockStack);

//Hot spot
HOTSPOT* StackInsertHotSpot(HSTACK *hStack, int nHotSpotPos);
HOTSPOT* GetHotSpot(BLOCKINFO *lpBlockInfo, HWND hWnd, CHARRANGE64 *cr);
BOOL NextHotSpot(BLOCKINFO *lpBlockInfo, HWND hWnd, BOOL bPrev);
void StackResetHotSpot(BLOCKINFO *lpBlockInfo);
void StackFreeHotSpot(HSTACK *hStack);

//Document words
void StackFillDocWord(HDELIMSTACK *hDelimiterStack, HDOCWORDS *hDocWordsStack, const wchar_t *wpTitlePart, int nTitlePartLen);
DOCWORDINFO* StackInsertDocWord(HDOCWORDS *hStack, wchar_t *wpWordDoc, int nWordDocLen);
DOCWORDINFO* StackGetDocWord(HDOCWORDS *hStack, const wchar_t *wpDocWord, int nDocWordLen);
void StackFreeDocWord(HDOCWORDS *hStack);

//Options
void ReadAutoCompleteOptions(HANDLE hOptions);
void SaveAutoCompleteOptions(HANDLE hOptions, DWORD dwFlags);
void InitAutoComplete();
void UninitAutoComplete();


//// Global variables

extern HSYNTAXFILE *lpSyntaxFileAutoComplete;

#endif
