#ifndef _HIGHLIGHT_H_
#define _HIGHLIGHT_H_


//// Defines

#define STRID_IGNORE_FONTSTYLE_GROUP 101
#define STRID_NORMAL                 103
#define STRID_BOLD                   104
#define STRID_ITALIC                 105
#define STRID_BOLDITALIC             106
#define STRID_AUTOMARK_GROUP         107
#define STRID_SYMBOLS                108
#define STRID_WORDS                  109
#define STRID_TEXTCOLOR              110
#define STRID_BGCOLOR                111

#define DLLA_HIGHLIGHT_MARKSELECTION       2
#define DLLA_HIGHLIGHT_UNMARKSELECTION     3
#define DLLA_HIGHLIGHT_CHECKMARKSELECTION  11

#define FF_FONTSTYLE 0x1
#define FF_FONTSIZE  0x2
#define FF_FONTFACE  0x4

//Ignore font style
#define IFS_FONTNORMAL             0x1  //Normal style.
#define IFS_FONTBOLD               0x2  //Bold style.
#define IFS_FONTITALIC             0x4  //Italic style.
#define IFS_FONTBOLDITALIC         0x8  //Bold italic style.

#define MARKID_TEXT    1
#define MARKID_AUTO    (DWORD)-1

#define MARKAUTO_NONE    0
#define MARKAUTO_SYMBOLS 1
#define MARKAUTO_WORDS   2

#define MARKMAX_WORD   1024


//// Structures

typedef struct _WORDINFO {
  struct _WORDINFO *next;
  struct _WORDINFO *prev;
  wchar_t *wpWord;
  int nWordLen;
  DWORD dwFlags;
  DWORD dwFontStyle;
  DWORD dwColor1;
  DWORD dwColor2;
} WORDINFO;

typedef struct _QUOTEINFO {
  struct _QUOTEINFO *next;
  struct _QUOTEINFO *prev;
  wchar_t *wpQuoteStart;
  int nQuoteStartLen;
  wchar_t *wpQuoteEnd;
  int nQuoteEndLen;
  wchar_t wchEscape;
  wchar_t *wpQuoteInclude;
  int nQuoteIncludeLen;
  wchar_t *wpQuoteExclude;
  int nQuoteExcludeLen;
  DWORD dwFlags;
  DWORD dwFontStyle;
  DWORD dwColor1;
  DWORD dwColor2;
} QUOTEINFO;

typedef struct _HWINDOWEDIT {
  struct _HWINDOWEDIT *next;
  struct _HWINDOWEDIT *prev;
  HWND hWndEdit;
  AEHDOC hDocEdit;
  BOOL bOldFont;
  DWORD dwFontFlags;
  LOGFONTW lfFontNew;
  LOGFONTW lfFontOld;
  BOOL bOldColors;
  AECOLORS aecNew;
  AECOLORS aecOld;
  HSTACK hMarkTextsStack;
} HWINDOWEDIT;

typedef struct _MARKTEXT {
  struct _MARKTEXT *next;
  struct _MARKTEXT *prev;
  AEHMARKTEXT hMarkTextHandle;
  DWORD dwMarkID;
} MARKTEXT;


//// Prototypes

BOOL CALLBACK HighLightSetupDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK HighLightEditProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void CALLBACK HighLightEditProcRet(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK HighLightEditParentMessages(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void PaintRichEditA(HSYNTAXFILE *lpSyntaxFile, HWND hWnd, RECT *rcUpdateRect);
void PaintRichEditW(HSYNTAXFILE *lpSyntaxFile, HWND hWnd, RECT *rcUpdateRect);
WORDINFO* StackInsertWord(HWORDSTACK *hStack, int nWordLen);
WORDINFO* StackGetWord(HWORDSTACK *hStack, wchar_t *wpWord, int nWordLen);
void StackFreeWord(HWORDSTACK *hStack);
QUOTEINFO* StackInsertQuote(HQUOTESTACK *hStack, int nQuoteStartLen);
void StackFreeQuote(HQUOTESTACK *hStack);
HWINDOWEDIT* StackInsertHighLightWindow(HSTACK *hStack);
HWINDOWEDIT* StackGetHighLightWindow(HSTACK *hStack, HWND hWndMaster, AEHDOC hDocMaster);
void StackDeleteHighLightWindow(HSTACK *hStack, HWINDOWEDIT *lpWindowEdit);
void StackFreeHighLightWindow(HSTACK *hStack);
MARKTEXT* StackInsertMarkText(HWINDOWEDIT *lpWindowEdit);
MARKTEXT* StackGetMarkText(HWINDOWEDIT *lpWindowEdit, DWORD dwMarkID);
BOOL StackIsAnyMarkText(HWINDOWEDIT *lpWindowEdit);
void StackDeleteMarkText(HWINDOWEDIT *lpWindowEdit, MARKTEXT *lpMarkText);
void StackFreeMarkText(HWINDOWEDIT *lpWindowEdit);
BOOL MarkSelection(HWINDOWEDIT *lpWindowEdit, const wchar_t *wpText, int nTextLen, DWORD dwColorText, DWORD dwColorBk, BOOL bMatchCase, DWORD dwFontStyle, DWORD dwMarkID);
BOOL UnmarkSelection(HWINDOWEDIT *lpWindowEdit, DWORD dwMarkID);
void CreateEditTheme(HSYNTAXFILE *lpSyntaxFile, HWND hWnd);
void GetFontAndColors(HSYNTAXFILE *lpSyntaxFile, HWINDOWEDIT *lpWindowEdit);
void RestoreFontAndColors(HWINDOWEDIT *lpWindowEdit);
void UnassignTheme(HWND hWnd);
void UpdateHighLight(HWND hWnd);
void UpdateHighLightAll();
DWORD GetFontStyle(DWORD dwFontStyle);
DWORD GetColorsToRestore(HWND hWnd, AECOLORS *aecHighlight);
COLORREF GetColorValueFromStrA(char *pColor);
COLORREF GetColorValueFromStrW(wchar_t *wpColor);
char* GetColorStrFromValueA(COLORREF crColor, char *szColor);
wchar_t* GetColorStrFromValueW(COLORREF crColor, wchar_t *wszColor);
void ReadHighLightOptions(HANDLE hOptions);
void SaveHighLightOptions(HANDLE hOptions, DWORD dwFlags);
void InitHighLight();
void UninitHighLight();


//// Global variables

extern HSTACK hHighLightWindowsStack;
extern HSTACK hMarkThemeStack;
extern DWORD dwIgnoreFontStyle;
extern DWORD dwAutoMarkFlags;
extern DWORD dwAutoMarkFontStyle;
extern DWORD dwAutoMarkTextColor;
extern DWORD dwAutoMarkBkColor;
extern BOOL bUpdateTheme;

#endif
